@@ -22,7 +22,10 @@
  *
  * The table structure can be created by the scripts/sql/nag.sql script.
  *
- * $Horde: nag/lib/Driver/sql.php,v 1.60.2.6 2005/06/23 03:08:53 selsky Exp $
+ * $Horde: nag/lib/Driver/sql.php,v 1.60.2.17 2006/05/05 16:04:39 jan Exp $
+ *
+ * See the enclosed file COPYING for license information (GPL).  If you
+ * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Jon Parise <jon@horde.org>
  * @since   Nag 0.1
@@ -45,10 +48,10 @@ class Nag_Driver_sql extends Nag_Driver {
     var $_connected = false;
 
     /**
-     * Construct a new SQL storage object.
+     * Constructs a new SQL storage object.
      *
      * @param string $tasklist  The tasklist to load.
-     * @param array  $params    A hash containing connection parameters.
+     * @param array $params     A hash containing connection parameters.
      */
     function Nag_Driver_sql($tasklist, $params = array())
     {
@@ -57,7 +60,7 @@ class Nag_Driver_sql extends Nag_Driver {
     }
 
     /**
-     * Retrieve one task from the database.
+     * Retrieves one task from the database.
      *
      * @param string $taskId  The id of the task to retrieve.
      *
@@ -88,13 +91,16 @@ class Nag_Driver_sql extends Nag_Driver {
         if (is_a($row, 'PEAR_Error')) {
             return $row;
         }
+        if ($row === null) {
+            return PEAR::raiseError(_("Not found"));
+        }
 
         /* Decode and return the task. */
         return $this->_buildTask($row);
     }
 
     /**
-     * Retrieve one task from the database by UID.
+     * Retrieves one task from the database by UID.
      *
      * @param string $uid  The UID of the task to retrieve.
      *
@@ -124,6 +130,9 @@ class Nag_Driver_sql extends Nag_Driver {
         if (is_a($row, 'PEAR_Error')) {
             return $row;
         }
+        if ($row === null) {
+            return PEAR::raiseError(_("Not found"));
+        }
 
         /* Decode and return the task. */
         $this->_tasklist = $row['task_owner'];
@@ -131,7 +140,7 @@ class Nag_Driver_sql extends Nag_Driver {
     }
 
     /**
-     * Add a task to the backend storage.
+     * Adds a task to the backend storage.
      *
      * @param string $name        The name (short) of the task.
      * @param string $desc        The description (long) of the task.
@@ -144,14 +153,14 @@ class Nag_Driver_sql extends Nag_Driver {
      *
      * @return string  The Nag ID of the new task.
      */
-    function add($name, $desc, $due = 0, $priority = 0, $completed = 0,
-                 $category = '', $alarm = 0, $uid = null)
+    function _add($name, $desc, $due = 0, $priority = 0, $completed = 0,
+                  $category = '', $alarm = 0, $uid = null)
     {
         /* Make sure we have a valid database connection. */
         $this->_connect();
 
         $taskId = md5(uniqid(mt_rand(), true));
-        if (is_null($uid)) {
+        if ($uid === null) {
             $uid = $this->generateUID();
         }
 
@@ -166,11 +175,11 @@ class Nag_Driver_sql extends Nag_Driver {
                         String::convertCharset($name, NLS::getCharset(), $this->_params['charset']),
                         String::convertCharset($uid, NLS::getCharset(), $this->_params['charset']),
                         String::convertCharset($desc, NLS::getCharset(), $this->_params['charset']),
-                        $due,
-                        $priority,
-                        $completed,
+                        (int)$due,
+                        (int)$priority,
+                        (int)$completed,
                         String::convertCharset($category, NLS::getCharset(), $this->_params['charset']),
-                        $alarm);
+                        (int)$alarm);
 
         /* Log the query at a DEBUG log level. */
         Horde::logMessage(sprintf('Nag_Driver_sql::add(): %s', $query),
@@ -185,15 +194,11 @@ class Nag_Driver_sql extends Nag_Driver {
             return $result;
         }
 
-        /* Log the creation of this item in the history log. */
-        $history = &Horde_History::singleton();
-        $history->log('nag:' . $this->_tasklist . ':' . $uid, array('action' => 'add'), true);
-
         return $taskId;
     }
 
     /**
-     * Modify an existing task.
+     * Modifies an existing task.
      *
      * @param string $taskId      The task to modify.
      * @param string $name        The name (short) of the task.
@@ -204,8 +209,8 @@ class Nag_Driver_sql extends Nag_Driver {
      * @param string $category    The category of the task.
      * @param integer $completed  The alarm associated with the task.
      */
-    function modify($taskId, $name, $desc, $due = 0, $priority = 0,
-                    $completed = 0, $category = '', $alarm = 0)
+    function _modify($taskId, $name, $desc, $due = 0, $priority = 0,
+                     $completed = 0, $category = '', $alarm = 0)
     {
         /* Make sure we have a valid database connection. */
         $this->_connect();
@@ -241,18 +246,11 @@ class Nag_Driver_sql extends Nag_Driver {
             return $result;
         }
 
-        /* Log the modification of this item in the history log. */
-        $task = $this->get($taskId);
-        if (!empty($task['uid'])) {
-            $history = &Horde_History::singleton();
-            $history->log('nag:' . $this->_tasklist . ':' . $task['uid'], array('action' => 'modify'), true);
-        }
-
         return true;
     }
 
     /**
-     * Move a task to a different tasklist.
+     * Moves a task to a different tasklist.
      *
      * @param string $taskId       The task to move.
      * @param string $newTasklist  The new tasklist.
@@ -280,7 +278,12 @@ class Nag_Driver_sql extends Nag_Driver {
         return true;
     }
 
-    function delete($taskId)
+    /**
+     * Deletes a task from the backend.
+     *
+     * @param string $taskId  The task to delete.
+     */
+    function _delete($taskId)
     {
         $this->_connect();
 
@@ -303,15 +306,12 @@ class Nag_Driver_sql extends Nag_Driver {
             return $result;
         }
 
-        /* Log the deletion of this item in the history log. */
-        if (!empty($task['uid'])) {
-            $history = &Horde_History::singleton();
-            $history->log('nag:' . $this->_tasklist . ':' . $task['uid'], array('action' => 'delete'), true);
-        }
-
         return true;
     }
 
+    /**
+     * Deletes all tasks from the backend.
+     */
     function deleteAll()
     {
         $this->_connect();
@@ -333,9 +333,12 @@ class Nag_Driver_sql extends Nag_Driver {
     /**
      * Retrieves tasks from the database.
      *
+     * @param integer $completed  Which tasks to retrieve (1 = all tasks,
+     *                            0 = incomplete tasks, 2 = complete tasks).
+     *
      * @return mixed  True on success, PEAR_Error on failure.
      */
-    function retrieve()
+    function retrieve($completed = 1)
     {
         /* Make sure we have a valid database connection. */
         $this->_connect();
@@ -344,6 +347,10 @@ class Nag_Driver_sql extends Nag_Driver {
         $query = sprintf('SELECT * FROM %s WHERE task_owner = ?',
                          $this->_params['table']);
         $values = array($this->_tasklist);
+        if ($completed != 1) {
+            $query .= ' AND task_completed = ?';
+            $values[] = $completed ? 1 : 0;
+        }
 
         /* Log the query at a DEBUG log level. */
         Horde::logMessage(sprintf('Nag_Driver_sql::retrieve(): %s', $query),
@@ -376,7 +383,7 @@ class Nag_Driver_sql extends Nag_Driver {
     }
 
     /**
-     * List all alarms near $date.
+     * Lists all alarms near $date.
      *
      * @param integer $date  The unix epoch time to check for alarms.
      *
@@ -413,6 +420,21 @@ class Nag_Driver_sql extends Nag_Driver {
 
     function _buildTask($row)
     {
+        /* Make sure tasks always have a UID. */
+        if (empty($row['task_uid'])) {
+            $row['task_uid'] = $this->generateUID();
+
+            $query = 'UPDATE ' . $this->_params['table'] .
+                ' SET task_uid = ?' .
+                ' WHERE task_owner = ? AND task_id = ?';
+            $values = array($row['task_uid'], $row['task_owner'], $row['task_id']);
+
+            /* Log the query at a DEBUG log level. */
+            Horde::logMessage(sprintf('Nag_Driver_sql adding missing UID: %s', $query),
+                              __FILE__, __LINE__, PEAR_LOG_DEBUG);
+            $this->_db->query($query, $values);
+        }
+
         /* Create a new task based on $row's values. */
         return array('tasklist_id' => $row['task_owner'],
                      'task_id' => $row['task_id'],
@@ -459,7 +481,16 @@ class Nag_Driver_sql extends Nag_Driver {
                 Horde::fatal($this->_db, __FILE__, __LINE__);
             }
 
-            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+            
+            // Set DB portability options.
+            switch ($this->_db->phptype) {
+            case 'mssql':
+                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
+                break;
+            default:
+                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+            }
+
 
             $this->_connected = true;
         }
