@@ -1,31 +1,54 @@
 <?php
 
-// Sort types
-/** @const NAG_SORT_NAME Sort by task name.           */ define('NAG_SORT_NAME', 'name');
-/** @const NAG_SORT_PRIORITY Sort by priority.        */ define('NAG_SORT_PRIORITY', 'priority');
-/** @const NAG_SORT_DUE Sort by due date.             */ define('NAG_SORT_DUE', 'due');
-/** @const NAG_SORT_COMPLETION Sort by completion.    */ define('NAG_SORT_COMPLETION', 'completed');
-/** @const NAG_SORT_CATEGORY Sort by category.        */ define('NAG_SORT_CATEGORY', 'category');
-/** @const NAG_SORT_OWNER Sort by owner.              */ define('NAG_SORT_OWNER', 'tasklist_id');
-
-/** @const NAG_SORT_ASCEND Sort in ascending order.   */ define('NAG_SORT_ASCEND', 0);
-/** @const NAG_SORT_DESCEND Sort in descending order. */ define('NAG_SORT_DESCEND', 1);
+/**
+ * Sort by task name.
+ */
+define('NAG_SORT_NAME', 'name');
 
 /**
- * Array for mapping the completed states to their string representations.
- * @var array $completion_map
+ * Sort by priority.
  */
-$GLOBALS['completion_map'] = array(
-    0 => _("No"),
-    1 => _("Yes"));
+define('NAG_SORT_PRIORITY', 'priority');
+
+/**
+ * Sort by due date.
+ */
+define('NAG_SORT_DUE', 'due');
+
+/**
+ * Sort by completion.
+ */
+define('NAG_SORT_COMPLETION', 'completed');
+
+/**
+ * Sort by category.
+ */
+define('NAG_SORT_CATEGORY', 'category');
+
+/**
+ * Sort by owner.
+ */
+define('NAG_SORT_OWNER', 'tasklist_id');
+
+/**
+ * Sort in ascending order.
+ */
+define('NAG_SORT_ASCEND', 0);
+
+/**
+ * Sort in descending order.
+ */
+define('NAG_SORT_DESCEND', 1);
 
 /**
  * Nag Base Class.
  *
- * $Horde: nag/lib/Nag.php,v 1.124 2004/12/20 15:41:29 jan Exp $
+ * $Horde: nag/lib/Nag.php,v 1.124.2.16 2006/05/25 01:56:01 chuck Exp $
+ *
+ * See the enclosed file COPYING for license information (GPL).  If you
+ * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Jon Parise <jon@horde.org>
- * @version $Revision: 1.124 $
  * @package Nag
  */
 class Nag {
@@ -38,15 +61,15 @@ class Nag {
         if ($hours > 1) {
             if ($minutes == 0) {
                 return sprintf(_("%d hours"), $hours);
-            } else if ($minutes == 1) {
+            } elseif ($minutes == 1) {
                 return sprintf(_("%d hours, %d minute"), $hours, $minutes);
             } else {
                 return sprintf(_("%d hours, %d minutes"), $hours, $minutes);
             }
-        } else if ($hours == 1) {
+        } elseif ($hours == 1) {
             if ($minutes == 0) {
                 return sprintf(_("%d hour"), $hours);
-            } else if ($minutes == 1) {
+            } elseif ($minutes == 1) {
                 return sprintf(_("%d hour, %d minute"), $hours, $minutes);
             } else {
                 return sprintf(_("%d hour, %d minutes"), $hours, $minutes);
@@ -54,7 +77,7 @@ class Nag {
         } else {
             if ($minutes == 0) {
                 return _("no time");
-            } else if ($minutes == 1) {
+            } elseif ($minutes == 1) {
                 return sprintf(_("%d minute"), $minutes);
             } else {
                 return sprintf(_("%d minutes"), $minutes);
@@ -66,13 +89,18 @@ class Nag {
      * Retrieves the current user's task list from storage.
      * This function will also sort the resulting list, if requested.
      *
-     * @param object Nag_Driver $storage   The current storage object.
-     * @param constant $sortby     (optional) The field by which to sort.
+     * @param Nag_Driver $storage  The current storage object.
+     * @param constant $sortby     The field by which to sort
      *                             (NAG_SORT_PRIORITY, NAG_SORT_NAME
-     *                              NAG_SORT_DUE, NAG_SORT_COMPLETION)
-     * @param constant $sortdir    (optional) The direction by which to sort.
-     *                             (NAG_SORT_ASC, NAG_SORT_DESC)
-     * @param constant $altsortby  (optional) The secondary sort field.
+     *                             NAG_SORT_DUE, NAG_SORT_COMPLETION).
+     * @param constant $sortdir    The direction by which to sort
+     *                             (NAG_SORT_ASC, NAG_SORT_DESC).
+     * @param constant $altsortby  The secondary sort field.
+     * @param array $tasklists     An array of tasklist to display or
+     *                             null/empty to display taskslists
+     *                             $GLOBALS['display_tasklists'].
+     * @param integer $completed   Which tasks to retrieve (1 = all tasks,
+     *                             0 = incomplete tasks, 2 = complete tasks).
      *
      * @return array  A list of the requested tasks.
      *
@@ -80,7 +108,9 @@ class Nag {
      */
     function listTasks($sortby = null,
                        $sortdir = null,
-                       $altsortby = null)
+                       $altsortby = null,
+                       $tasklists = null,
+                       $completed = null)
     {
         global $prefs, $registry;
 
@@ -93,7 +123,17 @@ class Nag {
         if (is_null($altsortby)) {
             $altsortby = $prefs->getValue('altsortby');
         }
+        if ($completed === null) {
+            $completed = $prefs->getValue('show_completed');
+        }
 
+        if (is_null($tasklists)) {
+            $tasklists = $GLOBALS['display_tasklists'];
+        }
+        if (!is_array($tasklists)) {
+            $tasklists = array($tasklists);
+        }
+        
         /* Sorting criteria for the task list. */
         $sort_functions = array(
             NAG_SORT_PRIORITY => 'ByPriority',
@@ -105,10 +145,12 @@ class Nag {
         );
 
         $tasks = array();
-        foreach ($GLOBALS['display_tasklists'] as $tasklist) {
+        $view_url = Horde::applicationUrl('view.php');
+        $task_url = Horde::applicationUrl('task.php');
+        foreach ($tasklists as $tasklist) {
             /* Create a Nag storage instance. */
             $storage = &Nag_Driver::singleton($tasklist);
-            $storage->retrieve();
+            $storage->retrieve($completed);
 
             /* Retrieve the tasklist from storage. */
             $newtasks = $storage->listTasks();
@@ -116,27 +158,26 @@ class Nag {
                 return $newtasks;
             }
 
+            $view_url_list = Util::addParameter($view_url, 'tasklist', $tasklist);
+            $task_url_list = Util::addParameter($task_url, 'tasklist', $tasklist);
             foreach ($newtasks as $taskID => $task) {
-                $url = Util::addParameter('view.php', 'task', $task['task_id']);
-                $url = Util::addParameter($url, 'tasklist', $task['tasklist_id']);
-                $newtasks[$taskID]['view_link'] = Horde::applicationUrl($url);
+                $newtasks[$taskID]['view_link'] = Util::addParameter($view_url_list, 'task', $task['task_id']);
 
-                $url = Util::addParameter('task.php', 'task', $task['task_id']);
-                $url = Util::addParameter($url, 'tasklist', $task['tasklist_id']);
-                $newtasks[$taskID]['complete_link'] = Horde::applicationUrl(Util::addParameter($url, 'actionID', 'complete_task'));
-                $newtasks[$taskID]['edit_link'] = Horde::applicationUrl(Util::addParameter($url, 'actionID', 'modify_task'));
-                $newtasks[$taskID]['delete_link'] = Horde::applicationUrl(Util::addParameter($url, 'actionID', 'delete_tasks'));
+                $task_url_task = Util::addParameter($task_url_list, 'task', $task['task_id']);
+                $newtasks[$taskID]['complete_link'] = Util::addParameter($task_url_task, 'actionID', 'complete_task');
+                $newtasks[$taskID]['edit_link'] = Util::addParameter($task_url_task, 'actionID', 'modify_task');
+                $newtasks[$taskID]['delete_link'] = Util::addParameter($task_url_task, 'actionID', 'delete_tasks');
             }
 
             $tasks = array_merge($tasks, $newtasks);
         }
 
-        /* We look for registered apis that support
-         * listAs(taskHash). */
+        /* We look for registered apis that support listAs(taskHash). */
         $apps = @unserialize($prefs->getValue('show_external'));
         if (is_array($apps)) {
             foreach ($apps as $app) {
-                if ($app != 'nag' && $registry->hasMethod('getListTypes', $app)) {
+                if ($app != 'nag' &&
+                    $registry->hasMethod('getListTypes', $app)) {
                     $types = $registry->callByPackage($app, 'getListTypes');
                     if (!empty($types['taskHash'])) {
                         $newtasks = $registry->callByPackage($app, 'listAs', array('taskHash'));
@@ -150,26 +191,6 @@ class Nag {
             }
         }
 
-        /* Filter complete/incomplete tasks if the user doesn't want
-         * them shown. */
-        switch ($prefs->getValue('show_completed')) {
-        case 0:
-            foreach ($tasks as $taskID => $task) {
-                if (!empty($task['completed'])) {
-                    unset($tasks[$taskID]);
-                }
-            }
-            break;
-
-        case 2:
-            foreach ($tasks as $taskID => $task) {
-                if (empty($task['completed'])) {
-                    unset($tasks[$taskID]);
-                }
-            }
-            break;
-        }
-
         /* Sort the array if we have a sort function defined for this
          * field. */
         if (isset($sort_functions[$sortby])) {
@@ -200,10 +221,37 @@ class Nag {
         return $storage->get($task);
     }
 
+    /**
+     * Returns the number of taks in task lists that the current user owns.
+     *
+     * @return integer  The number of tasks that the user owns.
+     */
+    function countTasks()
+    {
+        static $count;
+        if (isset($count)) {
+            return $count;
+        }
+
+        $tasklists = Nag::listTasklists(true, PERMS_ALL);
+
+        $count = 0;
+        foreach (array_keys($tasklists) as $tasklist) {
+            /* Create a Nag storage instance. */
+            $storage = &Nag_Driver::singleton($tasklist);
+            $storage->retrieve();
+
+            /* Retrieve the task list from storage. */
+            $count += count($storage->listTasks());
+        }
+
+        return $count;
+    }
+
     /**
      * Returns all the alarms active right on $date.
      *
-     * @param object $date         The start of the time range.
+     * @param object $date  The start of the time range.
      *
      * @return array  The alarms (alarmId) active on $date.
      */
@@ -235,11 +283,11 @@ class Nag {
     }
 
     /**
-     * List all task lists a user has access to.
+     * Lists all task lists a user has access to.
      *
-     * @param optional boolean $owneronly  Only return tasklists that this
-     *                                     user owns? Defaults to false.
-     * @param optional integer $permission The permission to filter tasklists by.
+     * @param boolean $owneronly  Only return tasklists that this user owns?
+     *                            Defaults to false.
+     * @param integer $permission The permission to filter tasklists by.
      *
      * @return array  The task lists.
      */
@@ -255,7 +303,7 @@ class Nag {
     }
 
     /**
-     * Get the default tasklist for the current user at the specified
+     * Returns the default tasklist for the current user at the specified
      * permissions level.
      */
     function getDefaultTasklist($permission = PERMS_SHOW)
@@ -279,10 +327,10 @@ class Nag {
     /**
      * Builds the HTML for a priority selection widget.
      *
-     * @param string  $name         The name of the widget.
-     * @param integer $selected     (optional) The default selected priority.
+     * @param string $name       The name of the widget.
+     * @param integer $selected  The default selected priority.
      *
-     * @return string       The HTML <select> widget.
+     * @return string  The HTML <select> widget.
      */
     function buildPriorityWidget($name, $selected = -1)
     {
@@ -302,33 +350,24 @@ class Nag {
     /**
      * Builds the HTML for a task completion state widget.
      *
-     * @param string  $name         The name of the widget.
-     * @param integer $selected     (optional) The default completion state.
+     * @param string $name      The name of the widget.
+     * @param integer $checked  The default completion state.
      *
-     * @return string       The HTML <select> widget.
+     * @return string  HTML for a checkbox representing the completion state.
      */
-    function buildCompletionWidget($name, $selected = 0)
+    function buildCompletionWidget($name, $checked = 0)
     {
-        global $completion_map;
-
-        $html = "<select id=\"$name\" name=\"$name\">";
-
-        foreach ($completion_map as $completed => $name) {
-            $html .= "<option value=\"$completed\"";
-            $html .= ($completed == $selected) ? ' selected="selected">' : '>';
-            $html .= $name . '</option>';
-        }
-        $html .= "</select>\n";
-
-        return $html;
+        $name = htmlspecialchars($name);
+        return "<input type=\"checkbox\" id=\"$name\" name=\"$name\"" .
+            ($checked ? ' checked="checked"' : '') . ' />';
     }
 
     /**
      * Formats the given Unix-style date string.
      *
-     * @param string $unixdate     The Unix-style date value to format.
+     * @param string $unixdate  The Unix-style date value to format.
      *
-     * @return string       The formatted due date string.
+     * @return string  The formatted due date string.
      */
     function formatDate($unixdate = '')
     {
@@ -340,15 +379,15 @@ class Nag {
 
         return sprintf(_("%s at %s"),
                        strftime($prefs->getValue('date_format'), $unixdate),
-                       strftime($prefs->getValue('twentyFour') ? '%H:%M' : '%I:%M %P', $unixdate));
+                       strftime($prefs->getValue('twentyFour') ? '%H:%M' : '%I:%M %p', $unixdate));
     }
 
     /**
      * Returns the string representation of the given completion status.
      *
-     * @param int $completed    The completion value.
+     * @param int $completed  The completion value.
      *
-     * @return string       The HTML representation of $completed.
+     * @return string  The HTML representation of $completed.
      */
     function formatCompletion($completed)
     {
@@ -360,21 +399,22 @@ class Nag {
     /**
      * Returns a colored representation of a priority.
      *
-     * @param int $priority    The priority level.
+     * @param int $priority  The priority level.
      *
-     * @return string       The HTML representation of $priority.
+     * @return string  The HTML representation of $priority.
      */
     function formatPriority($priority)
     {
-        return '<span class="pri-' . (int)$priority . '">' . (int)$priority . '</span>';
+        return '<span class="pri-' . (int)$priority . '">' . (int)$priority .
+            '</span>';
     }
 
     /**
      * Returns the string matching the given alarm value.
      *
-     * @param int  $value       The alarm value in minutes.
+     * @param int $value  The alarm value in minutes.
      *
-     * @return string       The formatted alarm string.
+     * @return string  The formatted alarm string.
      */
     function formatAlarm($value)
     {
@@ -400,20 +440,50 @@ class Nag {
     }
 
     /**
-     * Build Nag's list of menu items.
+     * Returns the specified permission for the current user.
      *
-     * @access public
+     * @since Nag 2.1
+     *
+     * @param string $permission  A permission, currently only 'max_tasks'.
+     *
+     * @return mixed  The value of the specified permission.
+     */
+    function hasPermission($permission)
+    {
+        global $perms;
+
+        if (!$perms->exists('nag:' . $permission)) {
+            return true;
+        }
+
+        $allowed = $perms->getPermissions('nag:' . $permission);
+        if (is_array($allowed)) {
+            switch ($permission) {
+            case 'max_tasks':
+                $allowed = array_reduce($allowed, create_function('$a, $b', 'return max($a, $b);'), 0);
+                break;
+            }
+        }
+
+        return $allowed;
+    }
+
+    /**
+     * Build Nag's list of menu items.
      */
     function getMenu($returnType = 'object')
     {
-        global $conf, $registry, $browser, $print_link;
+        global $conf, $registry, $print_link;
 
         require_once 'Horde/Menu.php';
 
         $menu = &new Menu();
         $menu->add(Horde::applicationUrl('list.php'), _("_List Tasks"), 'nag.png', null, null, null, basename($_SERVER['PHP_SELF']) == 'index.php' ? 'current' : null);
-        if (Nag::getDefaultTasklist(PERMS_EDIT)) {
-            $menu->add(Horde::applicationUrl(Util::addParameter('task.php', 'actionID', 'add_task')), _("_New Task"), 'add.png');
+        if (Nag::getDefaultTasklist(PERMS_EDIT) &&
+            (!empty($conf['hooks']['permsdenied']) ||
+             Nag::hasPermission('max_tasks') === true ||
+             Nag::hasPermission('max_tasks') > Nag::countTasks())) {
+            $menu->add(Horde::applicationUrl(Util::addParameter('task.php', 'actionID', 'add_task')), _("_New Task"), 'add.png', null, null, null, Util::getFormData('task') ? '__noselection' : null);
         }
         $menu->add(Horde::applicationUrl('search.php'), _("_Search"), 'search.png', $registry->getImageDir('horde'));
 
@@ -427,8 +497,8 @@ class Nag {
         }
 
         /* Print. */
-        if ($conf['menu']['print'] && isset($print_link) && $browser->hasFeature('javascript')) {
-            $menu->add("javascript:popup('$print_link'); return false;", _("_Print"), 'print.png', $registry->getImageDir('horde'));
+        if ($conf['menu']['print'] && isset($print_link)) {
+            $menu->add($print_link, _("_Print"), 'print.png', $registry->getImageDir('horde'), '_blank', 'popup(this.href); return false;');
         }
 
         if ($returnType == 'object') {
@@ -470,8 +540,8 @@ class Nag {
             }
         }
 
-        // Check here for guest task lists so that we don't get
-        // multiple messages after redirects, etc.
+        // Check here for guest task lists so that we don't get multiple
+        // messages after redirects, etc.
         if (!Auth::getAuth() && !count(Nag::listTasklists())) {
             $notification->push(_("No task lists are available to guests."));
         }
@@ -480,17 +550,185 @@ class Nag {
         $notification->notify(array('listeners' => 'status'));
     }
 
+    /**
+     * Sends email notifications that a task has been added, edited, or
+     * deleted to users that want such notifications.
+     *
+     * @param string $action     The event action. One of "add", "edit", or
+     *                           "delete".
+     * @param string $tasklist   The tasklist of the task we are dealing with.
+     * @param string $name       The name (short) of the task.
+     * @param string $desc       The description (long) of the task.
+     * @param integer $due       The due date of the task.
+     * @param integer $priority  The priority of the task.
+     */
+    function sendNotification($action, $tasklist, $name, $desc, $due, $priority)
+    {
+        global $conf;
+
+        switch ($action) {
+        case 'add':
+            $subject = _("Task added:");
+            $notification_message = _("You requested to be notified when tasks are added to your tasklists.") . "\n\n" . _("The task \"%s\" has been added to \"%s\" tasklist, with a due date of: %s.");
+            break;
+
+        case 'edit':
+            $subject = _("Task modified:");
+            $notification_message = _("You requested to be notified when tasks are edited on your tasklists.") . "\n\n" . _("The task \"%s\" has been edited on \"%s\" tasklist, with a due date of: %s.");
+            break;
+
+        case 'delete':
+            $subject = _("Task deleted:");
+            $notification_message = _("You requested to be notified when tasks are deleted from your tasklists.") . "\n\n" . _("The task \"%s\" has been deleted from \"%s\" tasklist, with a due date of: %s.");
+            break;
+
+        default:
+            return PEAR::raiseError('Unknown event action: ' . $action);
+        }
+
+        require_once 'Horde/Group.php';
+        require_once 'Horde/Identity.php';
+        require_once 'Horde/MIME.php';
+        require_once 'Horde/MIME/Headers.php';
+        require_once 'Horde/MIME/Message.php';
+
+        $share = &$GLOBALS['nag_shares']->getShare($tasklist);
+        if (is_a($share, 'PEAR_Error')) {
+            return $share;
+        }
+
+        $groups = &Group::singleton();
+        $recipients = array();
+        $identity = &Identity::singleton();
+        $from = $identity->getDefaultFromAddress(true);
+
+        $owner = $share->get('owner');
+        if (Nag::_notificationPref($owner, 'owner')) {
+            $recipients[$owner] = true;
+        }
+
+        foreach ($share->listUsers(PERMS_READ) as $user) {
+            if (!isset($recipients[$user])) {
+                $recipients[$user] = Nag::_notificationPref($user, 'read', $tasklist);
+            }
+        }
+        foreach ($share->listGroups(PERMS_READ) as $group) {
+            $group = $groups->getGroupById($group);
+            if (is_a($group, 'PEAR_Error')) {
+                continue;
+            }
+            $group_users = $group->listAllUsers();
+            if (is_a($group_users, 'PEAR_Error')) {
+                Horde::logMessage($group_users, __FILE__, __LINE__, PEAR_LOG_ERR);
+                continue;
+            }
+            foreach ($group_users as $user) {
+                if (!isset($recipients[$user])) {
+                    $recipients[$user] = Nag::_notificationPref($user, 'read', $tasklist);
+                }
+            }
+        }
+
+        $addresses = array();
+        foreach ($recipients as $user => $send) {
+            if ($send) {
+                $identity = &Identity::singleton('none', $user);
+                $email = $identity->getValue('from_addr');
+                if (strstr($email, '@')) {
+                    list($mailbox, $host) = explode('@', $email);
+                    $addresses[] = MIME::rfc822WriteAddress($mailbox, $host, $identity->getValue('fullname'));
+                }
+            }
+        }
+
+        if (!count($addresses)) {
+            return;
+        }
+
+        $msg_headers = &new MIME_Headers();
+        $msg_headers->addMessageIdHeader();
+        $msg_headers->addAgentHeader();
+        $msg_headers->addHeader('Date', date('r'));
+        $msg_headers->addHeader('From', $from);
+        $msg_headers->addHeader('Subject', $subject . ' ' . $name);
+
+        $message = "\n" . sprintf($notification_message, $name, $share->get('name'), $due ? strftime('%x %X', $due) : 'no due date' . "\n\n" . $desc);
+
+        $mime = &new MIME_Message();
+        $body = &new MIME_Part('text/plain', String::wrap($message, 76, "\n"), NLS::getCharset());
+
+        $mime->addPart($body);
+        $msg_headers->addMIMEHeaders($mime);
+
+        $mail_driver = $conf['mailer']['type'];
+        $mail_params = $conf['mailer']['params'];
+        if ($mail_driver == 'smtp' && $mail_params['auth'] &&
+            empty($mail_params['username'])) {
+            $mail_params['username'] = Auth::getAuth();
+            $mail_params['password'] = Auth::getCredential('password');
+        }
+
+        Horde::logMessage(sprintf('Sending event notifications for %s to %s', $name, implode(', ', $addresses)), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        return $mime->send(implode(', ', $addresses), $msg_headers, $mail_driver, $mail_params);
+    }
+
+    /**
+     * Returns whether a user wants email notifications for a tasklist.
+     *
+     * @access private
+     *
+     * @todo This method is causing a memory leak somewhere, noticeable if
+     *       importing a large amount of events.
+     *
+     * @param string $user      A user name.
+     * @param string $mode      The check "mode". If "owner", the method checks
+     *                          if the user wants notifications only for
+     *                          tasklists he owns. If "read", the method checks
+     *                          if the user wants notifications for all
+     *                          tasklists he has read access to, or only for
+     *                          shown tasklists and the specified tasklist is
+     *                          currently shown.
+     * @param string $tasklist  The name of the tasklist if mode is "read".
+     *
+     * @return boolean  True if the user wants notifications for the tasklist.
+     */
+    function _notificationPref($user, $mode, $tasklist = null)
+    {
+        $prefs = &Prefs::singleton($GLOBALS['conf']['prefs']['driver'],
+                                   'nag', $user, '', null,
+                                   false);
+        $prefs->retrieve();
+
+        $notification = $prefs->getValue('task_notification');
+        switch ($notification) {
+        case 'owner':
+            return $mode == 'owner';
+        case 'read':
+            return $mode == 'read';
+        case 'show':
+            if ($mode == 'read') {
+                $display_tasklists = unserialize($prefs->getValue('display_tasklsts'));
+                return in_array($tasklist, $display_tasklists);
+            }
+        }
+
+        return false;
+    }
+
     /**
      * Comparison function for sorting tasks by priority.
      *
      * @param array $a  Task one.
      * @param array $b  Task two.
      *
-     * @return integer  1 if task one is greater, -1 if task two is greater; 0 if they are equal.
+     * @return integer  1 if task one is greater, -1 if task two is greater;
+     *                  0 if they are equal.
      */
     function _sortByPriority($a, $b)
     {
-        if ($a['priority'] == $b['priority']) return 0;
+        if ($a['priority'] == $b['priority']) {
+            return 0;
+        }
         return ($a['priority'] > $b['priority']) ? 1 : -1;
     }
 
@@ -500,11 +738,14 @@ class Nag {
      * @param array $a  Task one.
      * @param array $b  Task two.
      *
-     * @return integer  -1 if task one is greater, 1 if task two is greater; 0 if they are equal.
+     * @return integer  -1 if task one is greater, 1 if task two is greater;
+     *                  0 if they are equal.
      */
     function _rsortByPriority($a, $b)
     {
-        if ($a['priority'] == $b['priority']) return 0;
+        if ($a['priority'] == $b['priority']) {
+            return 0;
+        }
         return ($a['priority'] > $b['priority']) ? -1 : 1;
     }
 
@@ -514,7 +755,8 @@ class Nag {
      * @param array $a  Task one.
      * @param array $b  Task two.
      *
-     * @return integer  1 if task one is greater, -1 if task two is greater; 0 if they are equal.
+     * @return integer  1 if task one is greater, -1 if task two is greater;
+     *                  0 if they are equal.
      */
     function _sortByName($a, $b)
     {
@@ -527,7 +769,8 @@ class Nag {
      * @param array $a  Task one.
      * @param array $b  Task two.
      *
-     * @return integer  -1 if task one is greater, 1 if task two is greater; 0 if they are equal.
+     * @return integer  -1 if task one is greater, 1 if task two is greater;
+     *                  0 if they are equal.
      */
     function _rsortByName($a, $b)
     {
@@ -540,11 +783,13 @@ class Nag {
      * @param array $a  Task one.
      * @param array $b  Task two.
      *
-     * @return integer  1 if task one is greater, -1 if task two is greater; 0 if they are equal.
+     * @return integer  1 if task one is greater, -1 if task two is greater;
+     *                  0 if they are equal.
      */
     function _sortByCategory($a, $b)
     {
-        return strcasecmp($b['category'], $a['category']);
+        return strcasecmp($a['category'] ? $a['category'] : _("Unfiled"),
+                          $b['category'] ? $b['category'] : _("Unfiled"));
     }
 
     /**
@@ -553,11 +798,13 @@ class Nag {
      * @param array $a  Task one.
      * @param array $b  Task two.
      *
-     * @return integer  -1 if task one is greater, 1 if task two is greater; 0 if they are equal.
+     * @return integer  -1 if task one is greater, 1 if task two is greater;
+     *                  0 if they are equal.
      */
     function _rsortByCategory($a, $b)
     {
-        return strcasecmp($b['category'], $a['category']);
+        return strcasecmp($b['category'] ? $b['category'] : _("Unfiled"),
+                          $a['category'] ? $a['category'] : _("Unfiled"));
     }
 
     /**
@@ -566,7 +813,8 @@ class Nag {
      * @param array $a  Task one.
      * @param array $b  Task two.
      *
-     * @return integer  1 if task one is greater, -1 if task two is greater; 0 if they are equal.
+     * @return integer  1 if task one is greater, -1 if task two is greater;
+     *                  0 if they are equal.
      */
     function _sortByDue($a, $b)
     {
@@ -591,9 +839,8 @@ class Nag {
      * @param array $a  Task one.
      * @param array $b  Task two.
      *
-     * @return integer  -1 if task one is greater,
-     *                   1 if task two is greater,
-     *                   0 if they are equal.
+     * @return integer  -1 if task one is greater, 1 if task two is greater,
+     *                  0 if they are equal.
      */
     function _rsortByDue($a, $b)
     {
@@ -618,11 +865,14 @@ class Nag {
      * @param array $a  Task one.
      * @param array $b  Task two.
      *
-     * @return integer  1 if task one is greater, -1 if task two is greater; 0 if they are equal.
+     * @return integer  1 if task one is greater, -1 if task two is greater;
+     *                  0 if they are equal.
      */
     function _sortByCompletion($a, $b)
     {
-        if ($a['completed'] == $b['completed']) return 0;
+        if ($a['completed'] == $b['completed']) {
+            return 0;
+        }
         return ($a['completed'] > $b['completed']) ? -1 : 1;
     }
 
@@ -632,11 +882,14 @@ class Nag {
      * @param array $a  Task one.
      * @param array $b  Task two.
      *
-     * @return integer  -1 if task one is greater, 1 if task two is greater; 0 if they are equal.
+     * @return integer  -1 if task one is greater, 1 if task two is greater;
+     *                  0 if they are equal.
      */
     function _rsortByCompletion($a, $b)
     {
-        if ($a['completed'] == $b['completed']) return 0;
+        if ($a['completed'] == $b['completed']) {
+            return 0;
+        }
         return ($a['completed'] < $b['completed']) ? -1 : 1;
     }
 
@@ -646,7 +899,8 @@ class Nag {
      * @param array $a  Task one.
      * @param array $b  Task two.
      *
-     * @return integer  1 if task one is greater, -1 if task two is greater; 0 if they are equal.
+     * @return integer  1 if task one is greater, -1 if task two is greater;
+     *                  0 if they are equal.
      */
     function _sortByOwner($a, $b)
     {
@@ -672,7 +926,8 @@ class Nag {
      * @param array $a  Task one.
      * @param array $b  Task two.
      *
-     * @return integer  -1 if task one is greater, 1 if task two is greater; 0 if they are equal.
+     * @return integer  -1 if task one is greater, 1 if task two is greater;
+     *                  0 if they are equal.
      */
     function _rsortByOwner($a, $b)
     {
